#include "icon_manager.hpp"
#include "gzip_decompressor.hpp"

std::map<std::string, IconData> IconManager::loadedIcons;

void IconManager::loadIcon(const char* name, const unsigned char* compressedData, size_t compressedSize, size_t originalSize, uint width, uint height) {
    // Check if icon is already loaded
    if (loadedIcons.find(name) != loadedIcons.end()) {
        return;
    }

    // Allocate memory for decompressed data
    uint8_t* decompressedData = new uint8_t[originalSize];
    if (!decompressedData) {
        Serial.printf("Failed to allocate memory for icon %s\n", name);
        return;
    }

    // Decompress the icon
    size_t decompressedSize = originalSize;
    if (!GzipDecompressor::decompressData(compressedData, compressedSize, decompressedData, &decompressedSize)) {
        Serial.printf("Failed to decompress icon %s\n", name);
        delete[] decompressedData;
        return;
    }

    Serial.printf("Decompressed icon %s to %zu bytes\n", name, decompressedSize);

    // Store the icon data
    IconData iconData = {
        .data = decompressedData,
        .width = width,
        .height = height
    };
    loadedIcons[name] = iconData;
}

void IconManager::unloadIcon(const char* name) {
    auto it = loadedIcons.find(name);
    if (it != loadedIcons.end()) {
        delete[] it->second.data;
        loadedIcons.erase(it);
    }
}

IconData* IconManager::getIcon(const char* name) {
    auto it = loadedIcons.find(name);
    if (it != loadedIcons.end()) {
        return &it->second;
    }
    return nullptr;
}

void IconManager::cleanup() {
    for (auto& pair : loadedIcons) {
        delete[] pair.second.data;
    }
    loadedIcons.clear();
} 